{
  "mcpServers": {
    "github": {
      "transport": "http",
      "url": "https://api.githubcopilot.com/mcp/",
      "headers": {
        "Authorization": {{ ("Bearer " + GITHUB_PERSONAL_ACCESS_TOKEN) | tojson }}
      }
    }
  }
}
