{
  "mcpServers": {
    "confluence": {
      "transport": "stdio",
      "command": "uvx",
      "args": ["mcp-atlassian"],
      "env": {
        "CONFLUENCE_URL": {{ CONFLUENCE_URL | tojson }},
        "CONFLUENCE_USERNAME": {{ CONFLUENCE_USERNAME | tojson }},
        "CONFLUENCE_API_TOKEN": {{ CONFLUENCE_API_TOKEN | tojson }}
      }
    }
  }
}
