{% extends "base.jinja" %}

{% block specific_requirements %}

{% if generation_prompt %}
### Generation Prompt
{{ generation_prompt }}
### End of Generation Prompt
{% endif %}

{% if additional_context %}
### Additional Context

Use the following additional context to inform your test generation. Incorporate relevant details from this context when crafting realistic prompts and scenarios:

{{ additional_context }}
### End of Additional Context
{% endif %}

{% endblock %}
