"""
Rhesis SDK Connector Module.

Internal implementation for bidirectional WebSocket communication.
This module is not meant to be imported directly by users.
"""

from rhesis.sdk.connector.manager import ConnectorManager

__all__ = ["ConnectorManager"]
